# -*- coding: utf-8 -*-
import os
import sys
import subprocess
import platform
from optparse import OptionParser
import ConfigParser
import logging

wapt_root_dir = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))

sys.path.insert(0, os.path.join(wapt_root_dir))
sys.path.insert(0, os.path.join(wapt_root_dir, 'lib'))
sys.path.insert(0, os.path.join(wapt_root_dir, 'lib', 'site-packages'))

from waptutils import *
from waptpackage import *

logger = logging.getLogger()
hdlr = logging.StreamHandler(sys.stdout)
hdlr.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)

listokpaquet = []

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


# retourner la dernière version disponible depuis le dépot principal
def best_version_for_package_in_repo(package, repotest, archi='all', os_version='force'):
    best_version = None
    packagefilename = None

    for package_entry in repotest.packages():
        if package_entry.package == package:
            for package_entry_2 in repotest.packages():
                if package_entry_2.package == package:

                    if Version(package_entry_2.max_os_version) > Version(os_version) or package_entry_2.max_os_version == '' or os_version == 'force':
                        if Version(package_entry.max_os_version) > Version(os_version) or package_entry.max_os_version == '' or os_version == 'force':

                            if Version(package_entry_2.min_os_version) < Version(os_version) or package_entry_2.min_os_version == '' or os_version == 'force':
                                if Version(package_entry.min_os_version) < Version(os_version) or package_entry.min_os_version == '' or os_version == 'force':
                                    if archi == package_entry.architecture or package_entry.architecture == "all" or archi == 'force':
                                        if archi == package_entry_2.architecture or package_entry_2.architecture == "all" or archi == 'force':
                                            if not package_entry.locale in ['','all','fr']:
                                                continue
                                            if not package_entry_2.locale in ['','all','fr']:
                                                continue
                                            if best_version == None:
                                                best_version = package_entry.version
                                                packagefilename = package_entry.filename
                                            if Version(best_version) < Version(package_entry_2.version):
                                                best_version = package_entry_2.version
                                                packagefilename = package_entry_2.filename
    return [best_version, packagefilename]


# Télécharge le paquet si le paquet s'il n'est pas présent dans le dépôt
def download_package_if_not_found(paquet, remote_repo_url_inner, local_repo_path_inner):
    if paquet is not None:
        if not os.path.isfile(local_repo_path_inner + paquet):
            try:
                print(local_repo_path_inner + paquet)
                print("[*]" + bcolors.WARNING + ' Download' +
                      remote_repo_url_inner + '//' + paquet + bcolors.ENDC)
                wget(remote_repo_url_inner + '//' + paquet, local_repo_path_inner)
                return True
            except:
                if os.path.isfile(paquet):
                    os.unlink(paquet)
                    raise
        else:
            return False
    return False


def update_package_repo(local_repository, remote_repository, local_repo_path_inner, remote_repo_url_inner):
    # Pour chaque paquet du dépôt local
    for local_package_entry in local_repository.packages():

        # pour chaque paquet du dépôt distant
        for remote_package_entry in remote_repository.packages():

            if not remote_package_entry.locale in ['','all','fr']:
                continue

            # si le nom du paquet cour est identique au dépôt distant
            if local_package_entry.package == remote_package_entry.package:

                # on emule pour les différent type d'archi pour les test de meilleur version
                for archi in ['x86', 'x64']:

                    # on recupère la meilleur version du dépôt local  et ditant pour l'archi de la boucle precedente et sans tester la version d'os
                    best_version_in_repo = best_version_for_package_in_repo(
                        remote_package_entry.package, remote_repository, archi=archi, os_version='force')
                    best_version_in_loc = best_version_for_package_in_repo(
                        local_package_entry.package, local_repository, archi=archi, os_version='force')
                    # si la meilleur version du dépôt distante et plus récente que la meilleur version du dépôt local on télécharge le paquet si il n'est pas présent
                    if Version(best_version_in_repo[1]) > Version(best_version_in_loc[1]):
                        download_package_if_not_found(
                            best_version_in_repo[1], remote_repo_url_inner, local_repo_path_inner)

                    # on recupère la meilleur version du dépôt local  et ditant pour l'archi de la boucle precedente et on test avec la version os du paquet distant en cour de test pour le max os version
                    best_version_in_repo = best_version_for_package_in_repo(
                        remote_package_entry.package, remote_repository, archi=archi, os_version=remote_package_entry.max_os_version)
                    best_version_in_loc = best_version_for_package_in_repo(
                        local_package_entry.package, local_repository, archi=archi, os_version=remote_package_entry.max_os_version)
                    # si la meilleur version du dépôt distante et plus récente que la meilleur version du dépôt local on télécharge le paquet si il n'est pas présent
                    if Version(best_version_in_repo[1]) > Version(best_version_in_loc[1]):
                        download_package_if_not_found(
                            best_version_in_repo[1], remote_repo_url_inner, local_repo_path_inner)

                    # on recupère la meilleur version du dépôt local  et ditant pour l'archi de la boucle precedente et on test avec la version os du paquet distant en cour de test pour le min os version
                    best_version_in_repo = best_version_for_package_in_repo(
                        remote_package_entry.package, remote_repository, archi=archi, os_version=remote_package_entry.min_os_version)
                    best_version_in_loc = best_version_for_package_in_repo(
                        local_package_entry.package, local_repository, archi=archi, os_version=remote_package_entry.min_os_version)
                    # si la meilleur version du dépôt distante et plus récente que la meilleur version du dépôt local on télécharge le paquet si il n'est pas présent
                    if Version(best_version_in_repo[1]) > Version(best_version_in_loc[1]):
                        download_package_if_not_found(
                            best_version_in_repo[1], remote_repo_url_inner, local_repo_path_inner)


def verify_dependencies(local_repository, remote_repository, local_repo_path_inner, remote_repo_url_inner):
    wgetrun = False
    for local_package_entry in local_repository.packages():
        for depends in ensure_list(local_package_entry.depends):
            found = False
            for test in local_repository.packages():
                if depends == test.package:
                    found = True
            if not found:
                remotepaquet = best_version_for_package_in_repo(
                    depends, remote_repository, archi='force')
                if remotepaquet[1] != None:
                    wgetrun = download_package_if_not_found(
                        remotepaquet[1], remote_repo_url_inner, local_repo_path_inner)
            found = False
    return wgetrun


def main():
    cfg = ConfigParser.ConfigParser()
    try:
    	cfg.read('/opt/wapt/conf/waptinfog.ini')
    except configparser.Error:
    	print "Can't parse config file : /opt/wapt/conf/waptinfog.ini"
    	quit()

    # Local repository settings
    local_repo_path = cfg.get('waptinfog','local_repo_path')
    remote_url = cfg.get('waptinfog', 'remote_url')
    admin = cfg.get('waptinfog', 'admin')
    password = cfg.get('waptinfog', 'password')

    remote_repo_url = 'https://%s:%s@%s/wapt' % (admin, password, remote_url)
    print("[*]" + bcolors.OKGREEN + " Start replication" + bcolors.ENDC)
    remoterepo = WaptRemoteRepo(name='main', url=remote_repo_url, timeout=4)
    localrepo = WaptLocalRepo(local_repo_path)

    print("[*]" + bcolors.OKGREEN + " Update local Packages file" + bcolors.ENDC)
    localrepo.update_packages_index(force_all=False)
    print("[*]" + bcolors.OKGREEN + " Updating repository" + bcolors.ENDC)

    update_package_repo(localrepo, remoterepo, local_repo_path, remote_repo_url)

    # relance de l'update package pour les test suivant
    print("[*]" + bcolors.OKGREEN + " Update local Packages file" + bcolors.ENDC)
    localrepo.update_packages_index(force_all=False)

    # Résoudre les dépendances manquante et les télécharger
    print("[*]" + bcolors.OKGREEN + " Verify dependencies" + bcolors.ENDC)

    while verify_dependencies(localrepo, remoterepo, local_repo_path, remote_repo_url):
        print("[*]" + bcolors.WARNING +
              " Dependencies not satisfied" + bcolors.ENDC)
        localrepo.update_packages_index(force_all=False)
        verify_dependencies(localrepo, remoterepo, local_repo_path, remote_repo_url)

    print("[*]" + bcolors.OKGREEN + " Updating repository" + bcolors.ENDC)
    update_package_repo(localrepo, remoterepo, local_repo_path, remote_repo_url)

    print("[*]" + bcolors.OKGREEN + " Update local Packages file" + bcolors.ENDC)
    localrepo.update_packages_index(force_all=False)
    localrepo = WaptLocalRepo(local_repo_path)

    for local_package_entry in localrepo.packages():

        for archi in ['x86', 'x64']:

            if not local_package_entry.locale in ['','all','fr']:
                continue


            best_version_in_loc = best_version_for_package_in_repo(local_package_entry.package, localrepo, archi=archi, os_version='force')
            if best_version_in_loc != None:
                listokpaquet.append(best_version_in_loc[1])

            best_version_in_loc = best_version_for_package_in_repo(
                local_package_entry.package, localrepo, archi=archi, os_version=local_package_entry.max_os_version)
            if best_version_in_loc != None:
                listokpaquet.append(best_version_in_loc[1])

            best_version_in_loc = best_version_for_package_in_repo(
                local_package_entry.package, localrepo, archi=archi, os_version=local_package_entry.min_os_version)
            if best_version_in_loc != None:
                listokpaquet.append(best_version_in_loc[1])

    print("[*]" + bcolors.OKGREEN + " Cleanup old packages" + bcolors.ENDC)
    for local_package_entry in localrepo.packages():
        found = False
        for filepaquet in listokpaquet:
            if local_package_entry.filename == filepaquet:
                found = True
        if not found:
            if os.path.isfile(local_repo_path + local_package_entry.filename):
                print('delete ' + local_repo_path + local_package_entry.filename)
                os.remove(local_repo_path + local_package_entry.filename)

    #print("[*]" + bcolors.OKGREEN + " Updating repository" + bcolors.ENDC)
    #update_package_repo(localrepo, remoterepo, local_repo_path, remote_repo_url)

    print("[*]" + bcolors.OKGREEN + " Update local Packages file" + bcolors.ENDC)
    localrepo.update_packages_index(force_all=False)
    os.system(r'chown -R wapt:www-data %s' % local_repo_path)

    print("[*]" + bcolors.OKGREEN +
          " Local repository up-to-date" + bcolors.ENDC)


if __name__ == "__main__":
    main()
